/*******************************************************************

  Copyright (c) 2006, Hyoung-Sun Kim.
  All Rights Reserved.

  You can contact us with
  web site <http://www.voiper.co.kr>
  e-mail <voiper@voiper.co.kr>

  Hyoung-Sun Kim.

  Distributed under the terms of the GNU General Public License

*******************************************************************/

/*

	<test.c>	2005-10-13,23:03

  : SIP Message Test Code

*/

#include "SipMessageMaker.h"



int main_make_test();
int main_message_test();
int main_header_test();


int main ()
{
	return main_make_test();
}





#define HS_TEST_CHECK_SIP_MESSAGE(tName)					\
if( tMessage==NULL )										\
{															\
	printf( "\n SipMessage Making Error "tName".\n\n" );	\
	return 5;												\
}															\
if( SipMessage_EncodeEx(tMessage,tString,&tPos,1000) != HS_OK )\
{															\
	printf( "\n SipMessage Encoding Error"tName".\n\n" );	\
	return 6;												\
}															\
else														\
	printf("\n\n%s\n\n",tString);							\
free(tMessage);												\
tMessage = NULL

static char *gSdp =
"\
v=0\r\n\
o=alice 2890844526 2890844526 IN IP4 client.atlanta.example.com\r\n\
s=-\r\n\
c=IN IP4 192.0.2.101\r\n\
t=0 0\r\n\
m=audio 49172 RTP/AVP 0\r\n\
a=rtpmap:0 PCMU/8000\r\n\
";

int main_make_test()
{
	IStack *tStack = NULL;
	IUa *tUa = NULL;
	IDialogue *tDialogue = NULL;
	ITransaction *tTransaction = NULL;

	ChainUnit *tChain = NULL;
	SipContact *tContact = NULL;
	SipContactUnit *tContactUnit = NULL;
	SipVia *tVia = NULL;

	SipMessage *tMessage = NULL;
	char tString[1024];

	if( (tStack=(IStack*)malloc(sizeof(IStack)))==NULL )
		return 1;
	if( (tUa=(IUa*)malloc(sizeof(IUa)))==NULL )
		return 1;
	if( (tDialogue=(IDialogue*)malloc(sizeof(IDialogue)))==NULL )
		return 1;
	if( (tTransaction=(ITransaction*)malloc(sizeof(ITransaction)))==NULL )
		return 1;

	new_IStack(tStack);
	new_IUa(tUa,tStack,120);
	new_IDialogue(tDialogue,tUa);
	new_ITransaction(tTransaction,tDialogue,FALSE,TRUE,FALSE,tDialogue->mCseqSeed++);

	/* stack setting
	*/
	NoLockList_AttachData(&(tStack->mUas),tUa);
	tStack->mMaxForward = 50;

	/* ua setting
	*/
	NoLockList_AttachData(&(tUa->mDialogues),tDialogue);

	strcpy(tUa->mPassword,"mypass");
	strcpy(tUa->mProxy,"local.proxy.com");
	tUa->mProxyPort = 6050;

	if( (tContact=(SipContact*)malloc(sizeof(SipContact)))==NULL )
	{
		delete_IStack(tStack);				free(tStack);
		return 2;
	}
	new_SipContact(tContact);

	if( (tContactUnit=(SipContactUnit*)malloc(sizeof(SipContactUnit)))==NULL )
	{
		delete_SipContact(tContact);		free(tContact);
		delete_IStack(tStack);				free(tStack);
		return 2;
	}
	new_SipContactUnit(tContactUnit,FALSE);

	tContactUnit->mDisplayName = SipStringAlloc("HSKIM");
	tContactUnit->mUri.mProtocol = SipStringAlloc(tStack->mProtocol);
	tContactUnit->mUri.mUserName = SipStringAlloc("030399466665");
	tContactUnit->mUri.mAddress = SipStringAlloc("local.ua.net");
	tContactUnit->mUri.mPort = SipStringAlloc("5060");

	NoLockList_AttachData(&(tContact->mContactUnits),tContactUnit);
	NoLockList_AttachData(&(tUa->mContacts),tContact);

	/* dialogue setting
	*/
	strcpy(tDialogue->mCallId,"1j9FpLxk3uxtm8tn@sip.mgcp.com");
	strcpy(tDialogue->mLocalTag,"ZZG00_local_876");
	strcpy(tDialogue->mRemoteTag,"ZZG00_remote_876");

	/* transaction setting
	*/
	tTransaction->mBranchId = SipStringAlloc("ZZ00branch876");
	if( (tVia=(SipVia*)malloc(sizeof(SipVia))) != NULL )
	{
		new_SipVia(tVia);
		sprintf(tString,"Via: SIP/%s/%s %s:%u; branch=%s\r\n",
			tStack->mVersion, tTransaction->mIsTcp? "TCP":"UDP", 
			"local.ua.net",5060, "z9hG4bKnashds7"
		);
		SipVia_Decode(tVia, tString, strlen(tString));
		NoLockList_AttachData(&(tTransaction->mVias),tVia);
	}
	tTransaction->mMethod = SipStringAlloc("CONSOL");

	/* add dialogue for call item
	*/
	tDialogue->mDestAddress = SipStringAlloc("remote.proxy.com");
	tDialogue->mDestDisplayName = SipStringAlloc("ICLEE");
	tDialogue->mDestPort = 3030;
	tDialogue->mDestUser = SipStringAlloc("01199261757");


	tMessage = MakeRequestRegister(tStack,tUa,tDialogue,tTransaction,NULL,NULL);
	HS_TEST_CHECK_SIP_MESSAGE("REGISTER");

	tMessage = MakeRequestInvite(tStack,tUa,tDialogue,tTransaction,"application/sdp",gSdp);
	HS_TEST_CHECK_SIP_MESSAGE("INVITE");

	tMessage = MakeRequestAck(tStack,tUa,tDialogue,tTransaction,NULL,NULL);
	HS_TEST_CHECK_SIP_MESSAGE("ACK");

	tMessage = MakeRequestBye(tStack,tUa,tDialogue,tTransaction,NULL,NULL);
	HS_TEST_CHECK_SIP_MESSAGE("BYE");

	tMessage = MakeRequestCancel(tStack,tUa,tDialogue,tTransaction,NULL,NULL);
	HS_TEST_CHECK_SIP_MESSAGE("CANCEL");

	tMessage = MakeResponseXxx(tStack,tUa,tDialogue,tTransaction,NULL,NULL,e_SipResponse_Trying);
	HS_TEST_CHECK_SIP_MESSAGE("Resp(Trying)");
	tMessage = MakeResponseXxx(tStack,tUa,tDialogue,tTransaction,NULL,NULL,e_SipResponse_Ringing);
	HS_TEST_CHECK_SIP_MESSAGE("Resp(Ringing)");
	tMessage = MakeResponseXxx(tStack,tUa,tDialogue,tTransaction,"application/sdp",gSdp,e_SipResponse_SessionProgress);
	HS_TEST_CHECK_SIP_MESSAGE("Resp(Session Progress)");
	tMessage = MakeResponseXxx(tStack,tUa,tDialogue,tTransaction,"application/sdp",gSdp,e_SipResponse_Ok);
	HS_TEST_CHECK_SIP_MESSAGE("Resp(OK)");
	tMessage = MakeResponseXxx(tStack,tUa,tDialogue,tTransaction,NULL,NULL,e_SipResponse_BusyHere);
	HS_TEST_CHECK_SIP_MESSAGE("Resp(Busy Here)");
	tMessage = MakeResponseXxx(tStack,tUa,tDialogue,tTransaction,NULL,NULL,e_SipResponse_RequestTerminated);
	HS_TEST_CHECK_SIP_MESSAGE("Resp(Request Terminated)");

	delete_IStack(tStack);
	free(tStack);
	return 0;
}





static char *gMsg = \
"\
REGISTER <sips:ss2.biloxi.example.com> SIP/2.0\r\n\
Via: SIP/2.0/TLS client.biloxi.example.com:5061;branch=z9hG4bKnashds7\r\n\
Max-Forwards: 70\r\n\
From: Bob <sips:bob@biloxi.example.com>;tag=a73kszlfl\r\n\
To: Bob <sips:bob@biloxi.example.com>\r\n\
Call-ID: 1j9FpLxk3uxtm8tn@biloxi.example.com\r\n\
CSeq: 1 REGISTER\r\n\
Contact: <sips:bob@client.biloxi.example.com>\r\n\
Content-Length: 0\r\n\r\n\
";
int main_message_test()
{
	int tLen = 0;
	HS_UINT tPos = 0;
	char tPacket[2048];
	SipMessage tMessage;

	new_SipMessage(&tMessage);

	if( SipMessage_Decode(&tMessage,gMsg,strlen(gMsg)+1) != HS_OK )
		printf("\n Decoding Error");
	else
	{
		printf("\n Decoding OK");
		if( SipMessage_EncodeEx(&tMessage,tPacket,&tPos,2040) != HS_OK )
			printf("\n Encoding Error");
		else
		{
			printf("\n Encoding OK");
			printf("\n\n%s\n\n", tPacket);
		}
	}

	delete_SipMessage(&tMessage);
	return 0;
}





int main_header_test()
{
	char tPacket[1024];
	char *tChar = NULL;
	HS_RESULT tRet = HS_OK;

	SipAccept tAccept;
	SipAlertInfo tAlertInfo;
	SipAuthorization tAuthorization;
	SipCallId tCallId;
	SipContact tContact;
	SipContactUnit *tContactUnit = NULL;

	/* header test - "Accept:", "Accept-Encoding:", "Accept-Language:"
	*/
	new_SipAccept(&tAccept);
	if( (tChar=SipStringAlloc("application/sdp")) != NULL )
		NoLockList_AttachData(&(tAccept.mList),tChar);
	if( (tChar=SipStringAlloc("application/t38fax")) != NULL )
		NoLockList_AttachData(&(tAccept.mList),tChar);
	SipAccept_Encode(&tAccept,tPacket,1000);
	printf(tPacket);
	SipAccept_Decode(&tAccept,
		"Accept: application/sdp;  level=1  ,   application/x-private  ,   text/html  \r\n",
		strlen("Accept: application/sdp; level=1 \t  ,   application/x-private  ,   text/html  \r\n")+1
	);
	SipAccept_Encode(&tAccept,tPacket,1000);
	printf(tPacket);

	/* header test - "Alert-Info: "
	*/
	new_SipAlertInfo(&tAlertInfo);
	tAlertInfo.mInfo=SipStringAlloc("<http://www.fromus.com/sounds/ring.wav>");
	SipAlertInfo_Encode(&tAlertInfo,tPacket,1000);
	printf(tPacket);
	SipAlertInfo_Decode(&tAlertInfo,
		"Alert-Info: <http://www.example.com/sounds/moo.wav>\r\n",
		strlen("Alert-Info: <http://www.example.com/sounds/moo.wav>\r\n")+1
	);
	SipAlertInfo_Encode(&tAlertInfo,tPacket,1000);
	printf(tPacket);

	/* header test - "Authorization: "
	*/
	new_SipAuthorization(&tAuthorization);
	tAuthorization.mType = SipStringAlloc("Digest");
	tAuthorization.mUserName = SipStringAlloc("Fromus");
	tAuthorization.mQop = SipStringAlloc("");
	tAuthorization.mNc = SipStringAlloc("0008");
	tAuthorization.mCnonce = SipStringAlloc("aaaaaaaaaaaaa");
	tAuthorization.mNonce = SipStringAlloc("ajfkdhasdldfshal");
	tAuthorization.mOpaque = SipStringAlloc("auth");
	tAuthorization.mUri = SipStringAlloc("www.fromus.com");
	tAuthorization.mResponse = SipStringAlloc("fjkas;djklasjfdka");
	tAuthorization.mStale = SipStringAlloc("TRUE");
	tAuthorization.mAlgorithm = SipStringAlloc("MD5");
	SipAuthorization_Encode(&tAuthorization,tPacket,1000,TRUE);
	printf(tPacket);
	SipAuthorization_Decode(&tAuthorization,
		"Authorization: Digest username=    Alice  , realm=\"atlanta.com\", cnonce=\"fdsf\", nc=\"0001\", nonce=\"84a4cc6f3082121f32b42a2187831a9e\", response=\"7587245234b3434cc3412213e5f113a5432\"\r\n",
		strlen("Authorization: Digest username=    Alice  , realm=\"atlanta.com\", cnonce=\"fdsf\", nc=\"0001\", nonce=\"84a4cc6f3082121f32b42a2187831a9e\", response=\"7587245234b3434cc3412213e5f113a5432\"\r\n")+1
	);
	SipAuthorization_Encode(&tAuthorization,tPacket,1000,TRUE);
	printf(tPacket);

	/* header test - "Call-ID: "
	*/
	new_SipCallId(&tCallId);
	tCallId.mInfo = SipStringAlloc( "djflajfdhak@fromus.co.kr" );
	SipCallId_Encode(&tCallId,tPacket,1000);
	printf(tPacket);
#if 0
	SipCallId_Decode(&tCallId,
		"Call-ID: f81d4fae-7dec-11d0-a765-00a0c91e6bf6@biloxi.com\r\n",
		strlen("Call-ID: f81d4fae-7dec-11d0-a765-00a0c91e6bf6@biloxi.com\r\n")+1
	);
#else
	SipCallId_Decode(&tCallId,
		"i:f81d4fae-7dec-11d0-a765-00a0c91e6bf6@192.0.2.4\r\n",
		strlen("i:f81d4fae-7dec-11d0-a765-00a0c91e6bf6@192.0.2.4\r\n")+1
	);
#endif
	SipCallId_Encode(&tCallId,tPacket,1000);
	printf(tPacket);

	new_SipContact(&tContact);
#if 0
	tContact.mIsStar = TRUE;
#else
	if( (tContactUnit=(SipContactUnit*)malloc(sizeof(SipContactUnit)))!=NULL )
	{
		new_SipContactUnit(tContactUnit,FALSE);
		tContactUnit->mDisplayName = SipStringAlloc("Kim Hyoung Sun");
		tContactUnit->mExpire = SipUintAlloc(3600);
		tContactUnit->mQ = SipStringAlloc("0.5");
		tContactUnit->mTag = SipStringAlloc("fsdafh37724Z");
		tContactUnit->mUri.mAddress = SipStringAlloc("local.fromus.co.kr");
		tContactUnit->mUri.mPort = SipUintAlloc(5070);
		tContactUnit->mUri.mProtocol = SipStringAlloc("https");
		tContactUnit->mUri.mUserName = SipStringAlloc("liotliol");
		tContactUnit->mUri.mVersion = SipStringAlloc("2.0");
		NoLockList_AttachData(&(tContact.mContactUnits),tContactUnit);
	}
	if( (tContactUnit=(SipContactUnit*)malloc(sizeof(SipContactUnit)))!=NULL )
	{
		new_SipContactUnit(tContactUnit,FALSE);
		tContactUnit->mDisplayName = SipStringAlloc("Sunny Kim");
		tContactUnit->mExpire = SipUintAlloc(600);
		tContactUnit->mQ = SipStringAlloc("0.7");
		tContactUnit->mTag = SipStringAlloc("TAG28399248Z");
		tContactUnit->mUri.mAddress = SipStringAlloc("fromus.co.kr");
		tContactUnit->mUri.mPort = SipUintAlloc(5070);
		tContactUnit->mUri.mProtocol = SipStringAlloc("mailto");
		tContactUnit->mUri.mUserName = SipStringAlloc("liotliol");
		tContactUnit->mUri.mLooseRoute = TRUE;
		NoLockList_AttachData(&(tContact.mContactUnits),tContactUnit);
	}
#endif/*0*/

	SipContact_Encode(&tContact,tPacket,1000);
	printf(tPacket);
#if 0
	SipContact_Decode(&tContact,
		"Contact: \"Mr. Watson\" sip:j.doe@big.com?subject=project  ;  lr  ;q=0.7; expires=3600, \"Mr. Watson\" <mailto:watson@bell-telephone.com> ;q=0.1\r\n",
		strlen("Contact: \"Mr. Watson\" sip:j.doe@big.com?subject=project  ;  lr  ;q=0.7; expires=3600, \"Mr. Watson\" <mailto:watson@bell-telephone.com> ;q=0.1\r\n")+1
	);
#else
	SipContact_Decode(&tContact,
		"Contact: *\r\n",
		strlen("Contact: *\r\n")+1
	);
#endif
	SipContact_Encode(&tContact,tPacket,1000);
	printf(tPacket);

	delete_SipAccept(&tAccept);
	delete_SipAlertInfo(&tAlertInfo);
	delete_SipAuthorization(&tAuthorization);
	delete_SipCallId(&tCallId);
	delete_SipContact(&tContact);
	return 0;
}