/*******************************************************************

  Copyright (c) 2006, Hyoung-Sun Kim.
  All Rights Reserved.

  You can contact us with
  web site <http://www.voiper.co.kr>
  e-mail <voiper@voiper.co.kr>

  Hyoung-Sun Kim.

  Distributed under the terms of the GNU General Public License

*******************************************************************/

/*

	<HSDefine.h>	2004-07-03,12:02

*/

#ifndef __HSDEFINE_H__
#define __HSDEFINE_H__





/* released vendor
*/
#define RELEASED_VENDOR		"voiper"





/* include base header
*/
#if defined(_WIN32) || defined(_WIN32_WCE)
	#include <malloc.h>
	#include <memory.h>
	#include <stdio.h>
	#include <string.h>
	#include <winsock.h>
	#define HS_SMALL_ENDIAN
#elif defined(_VXWORKS)
	#include "vxWorks.h"                    /* always first */
	#include "stdio.h"
	#include "string.h"
	#include "fioLib.h"
	#include <semLib.h>
	#include <taskLib.h>
	#include <string.h>

	#include "config.h"
	#include "usrConfig.h"
	#include "netLib.h"
	#include "sysLib.h"
	#define HS_BIG_ENDIAN
#endif





/* set debug range
*/
#define HS_DEBUG_ASN
#define HS_DEBUG_ASN_MEMBER
#define HS_DEBUG_ASN_PRINT





/* type definitions
*/
typedef char				HS_CHAR;
typedef unsigned char		HS_UCHAR;
#define HS_UCHAR_MAX		0xff
typedef short				HS_SHORT;
typedef unsigned short		HS_USHORT;
#define HS_USHORT_MAX		0xffff
typedef int					HS_INT;
typedef unsigned int		HS_UINT;
#define HS_UINT_MAX			0xffffffff

#if defined(_WIN32) || defined(_WIN32_WCE)
	typedef signed __int64		HS_INT64;
	typedef unsigned __int64	HS_UINT64;
#elif defined(_VXWORKS)
	typedef signed long long	HS_INT64;
	typedef unsigned long long	HS_UINT64;
#endif

typedef int					HS_RESULT;

#ifndef BOOL
#define BOOL				int
#endif
#ifndef FALSE
#define FALSE				0
#endif
#ifndef TRUE
#define TRUE				1
#endif
#ifndef NULL
#define NULL				0
#endif
#define HS_NULL				NULL

#define HS_QID				HS_UINT
#define HS_INVALID_QID		HS_UINT_MAX

#define HS_TID							HS_UINT
#define HS_TR_HANDLE					HS_UINT
#define HS_CALL_HANDLE					HS_UINT
#define HS_STACK_HANDLE					void*
#define HS_INVALID_STACK_HANDLE			NULL
#define HS_INVALID_TID					HS_UINT_MAX
#define HS_INVALID_POS					HS_UINT_MAX
#define HS_INVALID_HANDLE				HS_UINT_MAX
#define HS_INVALID_TSAP_PORT			HS_USHORT_MAX

#if !defined(INVALID_SOCKET)
typedef int SOCKET;
#define INVALID_SOCKET		-1
#define closesocket			close
#endif

#define HS_INVALID_SOCKET	INVALID_SOCKET





/* semaphore definitions
*/
#if defined(_WIN32) || defined(_WIN32_WCE)
	#define HSSemaphore				CRITICAL_SECTION
	#define HSOpenSemaphore(sem)	InitializeCriticalSection(&sem)
	#define HSCloseSemaphore(sem)	DeleteCriticalSection(&sem)
	#define HSLockSemaphore(sem)	EnterCriticalSection(&sem)
	#define HSUnlockSemaphore(sem)	LeaveCriticalSection(&sem)
#elif defined(_VXWORKS)
	#define HSSemaphore				SEM_ID
	#define HSOpenSemaphore(sem)	sem = semBCreate(SEM_Q_PRIORITY,SEM_FULL)
	#define HSCloseSemaphore(sem)	semTerminate(sem)
	#define HSLockSemaphore(sem)	semTake(sem,WAIT_FOREVER)
	#define HSUnlockSemaphore(sem)	semGive(sem)
#endif





/* base thread queue messages
*/
#define HS_QM								1120
#define HS_QM_USER							1200
#define HS_QM_MAX							2000
/* wparam: 0
   lparam: any value
*/
#define HS_QM_START							HS_QM +1
/* wparam: 0
   lparam: null
*/
#define HS_QM_CHECK							HS_QM_START +1
/* wparam: 0
   lparam: null
*/
#define HS_QM_STOP							HS_QM_CHECK +1
/* wparam: 0
   lparam: HSTimerEvent object pointer
*/
#define HS_QM_TIMER							HS_QM_STOP +1
/* wparam: 0
   lparam: QmConnected object pointer
*/
#define HS_QM_CONNECTED							HS_QM_TIMER +1





/* base return value
   (-): not ok, but not error.
   (0): every thing is ok.
   (+): error.
*/
#define HS_NOK								-1

#define HS_OK								0

#define HS_ERR								1
#define HS_ERR_NULL							2
#define HS_ERR_NULL_PARAM					3
#define HS_ERR_MALLOC						4
#define HS_ERR_INVALID						5
#define HS_ERR_INVALID_PARAM				6
#define HS_ERR_INVALID_VALUE				7
#define HS_ERR_CONFLICT						8
#define HS_ERR_NO_EXIST						9
#define HS_ERR_OVER							10
#define HS_ERR_EMPTY						11
#define HS_ERR_FILE							12

#define HS_ERR_NET_SOCKET					30
#define HS_ERR_NET_BIND						31
#define HS_ERR_NET_SEND						32
#define HS_ERR_NET_RECV						33

#define HS_ERR_NOT_SUPPORT_YET				100
#define HS_ERR_LICENCE						1000

#define HS_TRY(t)					{if( (tRet=t) != HS_OK ) return tRet;}
#define HS_TRY_OK(t)				{if(    t    != HS_OK ) return HS_OK;}





#endif/*__HSDEFINE_H__*/

